/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#ifndef _PLSFILE_MODEL_H_
#define _PLSFILE_MODEL_H_

#include <QVariant>
#include <QString>
#include <QStringList>
#include <QObject>

#include "MediaItem/mediaitem.h"

/*
********************************************************************************
*                                                                              *
*    Class PlsFileModel                                                        *
*                                                                              *
********************************************************************************
*/
class PlsFileModel : public QObject
{
Q_OBJECT
     static PlsFileModel*  INSTANCE;

  public:
     PlsFileModel(QObject *parent = 0);
     ~PlsFileModel();
     static PlsFileModel* instance() { return INSTANCE; }

     //! custom method
     MEDIA::MediaPtr rootItem();
     void clear();

     //! get Url method
     QList<MEDIA::TrackPtr> getItemChildrenTracks(const MEDIA::MediaPtr parent);

     //! filtering method
     bool matches(const QString text) const;
     bool isTrackFiltered(const MEDIA::TrackPtr track);
     bool isPlaylistFiltered(const MEDIA::PlaylistPtr playlistItem);
     void setFilter(const QString & f) {m_filter_pattern = f;}

     //! playing track
     void stopPlayingItem();
     void updatePlayingItem(MEDIA::TrackPtr);
     void updateBrokenItem(MEDIA::TrackPtr);

  signals:
    void signalPlayingStatusChanged();
    void signalFavoriteStatusChanged();
    void modelCleared();

  private:
     MEDIA::MediaPtr        m_rootItem;
     MEDIA::TrackPtr        m_playing_track;
     QString                m_filter_pattern;
};

#endif // _PLSFILE_MODEL_H_
