/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#ifndef _CONTEXT_SCENE_H_
#define _CONTEXT_SCENE_H_


#include "widget/context_widget.h"

#include <QWidget>
#include <QGraphicsWidget>
#include <QGraphicsLinearLayout>
#include <QGraphicsScene>
#include <QMap>

class EnginePlayer;
/*
********************************************************************************
*                                                                              *
*    Class ContextScene                                                        *
*                                                                              *
********************************************************************************
*/
class ContextScene : public QGraphicsScene
{
Q_OBJECT
  public:
    ContextScene(QWidget* parent);
    ~ContextScene();

    void setActivate(bool a) {m_activated = a;}
    bool isActivated() {return m_activated;}
    void startContextScene();

  private:
    void clear_scene(bool isplaying=true);
    void init_scene();
    bool analyse_new_media();

  private slots:
    void slot_update_draw();
    void slot_update_scene();

  private :
    QWidget                   *m_parentWidget;

    bool                       m_activated;
    bool                       m_initialized;

    /*   player  and states  */
    EnginePlayer              *m_engine_player;
    QString                    m_url;
    MEDIA::TrackPtr            m_current_track;
    bool                       is_new_artist;
    bool                       is_new_album;
    bool                       is_new_track;

    /*    providers          */
    QMap<QString,InfoProvider*>   m_map_providers;
    int                        m_request_id;

    /*    graphics items     */
    QGraphicsLinearLayout     *m_layout;
    QGraphicsWidget           *m_container;
    QGraphicsWidget           *m_container_now_playing;

    NowPlayingInfoWidget      *m_nowplaying_info_widget;
    ArtistInfoWidget          *m_artist_info_widget;
    ArtistSimilarWidget       *m_similar_info_widget;
    DiscoInfoWidget           *m_disco_info_widget;
    AlbumInfoWidget           *m_album_info_widget;
    LyricsInfoWidget          *m_lyrics_info_widget;
    Header                    *m_headeritem;

};


#endif // _CONTEXT_SCENE_H_
